@@ -155,6 +155,9 @@ struct _GtkFileChooserButtonPrivate
 
   /* Used for hiding/showing the dialog when the button is hidden */
   guint8 active                       : 1;
+
+  /* Used to track whether we need to set a default current folder on ::map() */
+  guint8 folder_has_been_set          : 1;
 };
 
 
@@ -211,6 +214,7 @@ static void     gtk_file_chooser_button_show_all           (GtkWidget        *wi
 static void     gtk_file_chooser_button_hide_all           (GtkWidget        *widget);
 static void     gtk_file_chooser_button_show               (GtkWidget        *widget);
 static void     gtk_file_chooser_button_hide               (GtkWidget        *widget);
+static void     gtk_file_chooser_button_map                (GtkWidget        *widget);
 static gboolean gtk_file_chooser_button_mnemonic_activate  (GtkWidget        *widget,
 							    gboolean          group_cycling);
 static void     gtk_file_chooser_button_style_set          (GtkWidget        *widget,
@@ -322,6 +326,7 @@ gtk_file_chooser_button_class_init (GtkFileChooserButtonClass * class)
   widget_class->hide_all = gtk_file_chooser_button_hide_all;
   widget_class->show = gtk_file_chooser_button_show;
   widget_class->hide = gtk_file_chooser_button_hide;
+  widget_class->map = gtk_file_chooser_button_map;
   widget_class->style_set = gtk_file_chooser_button_style_set;
   widget_class->screen_changed = gtk_file_chooser_button_screen_changed;
   widget_class->mnemonic_activate = gtk_file_chooser_button_mnemonic_activate;
@@ -593,6 +598,7 @@ gtk_file_chooser_button_constructor (GType                  type,
   GObject *object;
   GtkFileChooserButtonPrivate *priv;
   GSList *list;
+  char *current_folder;
 
   object = (*G_OBJECT_CLASS (gtk_file_chooser_button_parent_class)->constructor) (type,
 										  n_params,
@@ -627,6 +633,13 @@ gtk_file_chooser_button_constructor (GType                  type,
 					       -1);
     }
 
+  current_folder = gtk_file_chooser_get_current_folder_uri (GTK_FILE_CHOOSER (priv->dialog));
+  if (current_folder != NULL)
+    {
+      priv->folder_has_been_set = TRUE;
+      g_free (current_folder);
+    }
+
   g_free (priv->backend);
   priv->backend = NULL;
 
@@ -1013,6 +1026,28 @@ gtk_file_chooser_button_hide (GtkWidget *widget)
     (*GTK_WIDGET_CLASS (gtk_file_chooser_button_parent_class)->hide) (widget);
 }
 
+static void
+gtk_file_chooser_button_map (GtkWidget *widget)
+{
+  GtkFileChooserButtonPrivate *priv;
+
+  priv = GTK_FILE_CHOOSER_BUTTON_GET_PRIVATE (widget);
+
+  if (!priv->folder_has_been_set)
+    {
+      char *current_working_dir;
+
+      current_working_dir = g_get_current_dir ();
+      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (widget), current_working_dir);
+      g_free (current_working_dir);
+
+      priv->folder_has_been_set = TRUE;
+    }
+
+  if (GTK_WIDGET_CLASS (gtk_file_chooser_button_parent_class)->map)
+    (*GTK_WIDGET_CLASS (gtk_file_chooser_button_parent_class)->map) (widget);
+}
+
 static gboolean
 gtk_file_chooser_button_mnemonic_activate (GtkWidget *widget,
 					   gboolean   group_cycling)
@@ -1109,6 +1144,9 @@ change_icon_theme (GtkFileChooserButton *button)
       gtk_list_store_set (GTK_LIST_STORE (priv->model), &iter,
 			  ICON_COLUMN, pixbuf,
 			  -1);
+
+      if (pixbuf)
+	g_object_unref (pixbuf);
     }
   while (gtk_tree_model_iter_next (priv->model, &iter));
 
@@ -1311,7 +1349,8 @@ model_add_special (GtkFileChooserButton *button)
 			  DATA_COLUMN, path,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       button->priv->n_special++;
 
 #ifndef G_OS_WIN32
@@ -1339,7 +1378,8 @@ model_add_special (GtkFileChooserButton *button)
 			  DATA_COLUMN, path,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       button->priv->n_special++;
     }
 }
@@ -1379,7 +1419,8 @@ model_add_volumes (GtkFileChooserButton *button,
 			  DATA_COLUMN, volumes->data,
 			  -1);
 
-      g_object_unref (pixbuf);
+      if (pixbuf)
+	g_object_unref (pixbuf);
       g_free (display_name);
 
       button->priv->n_volumes++;
@@ -1953,10 +1994,21 @@ open_dialog (GtkFileChooserButton *button)
 
       if (GTK_WIDGET_TOPLEVEL (toplevel) && GTK_IS_WINDOW (toplevel))
         {
-          if (GTK_WINDOW (toplevel) != gtk_window_get_transient_for (GTK_WINDOW (priv->dialog)))
- 	    gtk_window_set_transient_for (GTK_WINDOW (priv->dialog),
-					  GTK_WINDOW (toplevel));
+	  GtkWindow *transient_parent = gtk_window_get_transient_for (GTK_WINDOW (priv->dialog));
+	  
+	  if (GTK_WINDOW (toplevel) != transient_parent)
+	    {
+	      if (transient_parent && transient_parent->group)
+		gtk_window_group_remove_window (transient_parent->group, 
+						GTK_WINDOW (priv->dialog));
 	      
+	      gtk_window_set_transient_for (GTK_WINDOW (priv->dialog),
+					    GTK_WINDOW (toplevel));
+	      if (GTK_WINDOW (toplevel)->group)
+		gtk_window_group_add_window (GTK_WINDOW (toplevel)->group, 
+					     GTK_WINDOW (priv->dialog));
+	    }
+	  
 	  gtk_window_set_modal (GTK_WINDOW (priv->dialog),
 				gtk_window_get_modal (GTK_WINDOW (toplevel)));
 	}
@@ -2058,7 +2110,15 @@ static void
 dialog_current_folder_changed_cb (GtkFileChooser *dialog,
 				  gpointer        user_data)
 {
-  g_signal_emit_by_name (user_data, "current-folder-changed");
+  GtkFileChooserButton *button;
+  GtkFileChooserButtonPrivate *priv;
+
+  button = GTK_FILE_CHOOSER_BUTTON (user_data);
+  priv = GTK_FILE_CHOOSER_BUTTON_GET_PRIVATE (button);
+
+  priv->folder_has_been_set = TRUE;
+
+  g_signal_emit_by_name (button, "current-folder-changed");
 }
 
 static void
@@ -2261,7 +2321,7 @@ gtk_file_chooser_button_new_with_backend (const gchar          *title,
  * 
  * Creates a #GtkFileChooserButton widget which uses @dialog as it's
  * file-picking window. Note that @dialog must be a #GtkFileChooserDialog (or
- * subclass).
+ * subclass) and must not have %GTK_DIALOG_DESTROY_WITH_PARENT set.
  * 
  * Returns: a new button widget.
  * 
