@@ -26,9 +26,9 @@
 
 #include <config.h>
 
-#include "gtkalias.h"
 #include "gtkmenu.h"
 #include "gtktearoffmenuitem.h"
+#include "gtkalias.h"
 
 #define ARROW_SIZE 10
 #define TEAR_LENGTH 5
@@ -157,7 +157,7 @@ gtk_tearoff_menu_item_paint (GtkWidget   *widget,
 	  gint selected_shadow_type;
 	  
 	  gtk_widget_style_get (widget,
-				"selected_shadow_type", &selected_shadow_type,
+				"selected-shadow-type", &selected_shadow_type,
 				NULL);
 	  gtk_paint_box (widget->style,
 			 widget->window,
@@ -285,3 +285,6 @@ gtk_tearoff_menu_item_parent_set (GtkWidget *widget,
 			tearoff_menu_item);
     }  
 }
+
+#define __GTK_TEAROFF_MENU_ITEM_C__
+#include "gtkaliasdef.c"
